CONST_CELSIUS = 0
CONST_FARENHEIT = 1

# Websocket opcodes
OP_CODE_PING = "2"
OP_CODE_PONG = "3"
OP_CODE_CONNECTED = "0"
OP_CODE_CONNECTED_ACK = "40"
OP_CODE_COMMAND = "42"
OP_CODE_COMMAND_RESPONSE = "43"
